#!/usr/bin/python

#
# Tom Winegar Subaru Telescope - winegar@naoj.org
# unpack.py - client download program from STARS Query
#
# instructions for downloading:
# 1. create {new directory}
# 2. download S2Query.tar from email into {new directory}
# 3. from {new directory} execute 'tar -xvf S2Query.tar'
# 4. from {new directory} execute './zadmin/unpack.py'
# 5. files will be download to {new directory} without progess bar or notice
# 6. monitor the downbload in another window using 'ls -al' to list or 'ls -al | wc -l' to count 
#
# instructions for MD5 check and Missing file check after download:
# 1. execute ./zadmin/unpack.py again
# 2. Confirm MD5s on Existing Files? = Y
#
# instructions for downloading MD5 Fails and Missing files after downloading:
# 1. execute ./zadmin/unpack.py
# 2. Download Missing/MD5Fails files ONLY? = Y
#
# For HSC Permanent Archive Queries
# 1. execute ./zadmin/unpack.py
# 2. choose network route
# 3. Update wget-Input file from STARS? = Y
#


import os,statvfs,shutil
import datetime
import subprocess
import os
import time

def BytestoText( diskfree ) :

	df=int(diskfree)/1000000
	dfs=str(df)+ 'MB'
	
	if df > 1000:
	
		df=int(diskfree)/1000000000
		dfs=str(df)+ ' GB'
				
		if df > 1000:
			df=int(diskfree)/1000000000000
			dfs=str(df)+ ' TB'
			
	return dfs


def wGetRunning( ) :

	foundText=""

	cmd  = "ps -ef | grep 'wget --user='"
	
	fv = os.popen(cmd).read().strip()

#	print("fv: \n"+fv)	
	pslines = fv.split("\n")
	wget_ps = len( pslines ) - 2

	
	found = False
	
	if wget_ps > 0:
	
		found = True
		foundText = fv
			
	return found, foundText

def getDomain( netfile ) :

	domain='none'
	
	if netfile == 'uhn.txt':
		domain = "http://stars2.naoj.hawaii.edu/data/"
	if netfile == 'uhns.txt':
		domain = "https://stars2.naoj.hawaii.edu/data/"
	if netfile == 'mtkn.txt' :
		domain = "http://www.mastars.nao.ac.jp/data/"
	if netfile == 'mtkns.txt' :
		domain = "https://www.mastars.nao.ac.jp/data/"
	if netfile == 'mtkin.txt' :
#		domain = "http://m2query.mastars.nao.ac.jp/data/"
		domain = "http://mquery.mastars.nao.ac.jp/data/"
	if netfile == 'oc3n.txt' :
		domain =  "http://squery.subaru.nao.ac.jp/data/"
	if netfile == 'oc3ns.txt' :
		domain =  "https://squery.subaru.nao.ac.jp/data/"


	if netfile == 'zip-uhn.txt':
		domain = "http://stars2.naoj.hawaii.edu/data/"
	if netfile == 'zip-uhns.txt':
		domain = "https://stars2.naoj.hawaii.edu/data/"

	if netfile == 'zip-oc3n.txt' :
		domain =  "http://squery.subaru.nao.ac.jp/data/"
	if netfile == 'zip-oc3ns.txt' :
		domain =  "https://squery.subaru.nao.ac.jp/data/"
	
	return ( domain )

def checkMD5s( qdir, netfile, quser, qpwd, mnocheck ) :

	cwd=os.getcwd()
	
	response10=raw_input('Update MD5s values from STARS? (y/n): ')
	
	if response10 == 'y' or response10 == 'Y' :

# change dir to /zadmin for the netfile download					
		newdir=cwd + '/zadmin/'

		os.chdir(newdir)
		
		domain = getDomain( netfile )
		if not domain == 'none' :				
			netfileInput = domain + qdir + "/zadmin/zadmin/filelist.txt"											
			cmd='wget -N --user='+quser+' --password='+qpwd+mnocheck+ " " +netfileInput+" -a ./transfer.log"
			print( "MD5 update filelist.txt: "+cmd )
			os.system(cmd)
		else:
			print( "Domain not found: " + netfile )

# change dir back											
		os.chdir(cwd)
	

#	listpath30 = cwd+'/zadmin/zadmin/filelist.txt'
	listpath30 = cwd+'/zadmin/filelist.txt'

	missCount = 0
	failCount = 0
	goodCount = 0
	zipfzCount = 0
	fzCount = 0
	fitsCount = 0

	if os.path.exists( listpath30 ):

		# open filelist.txt
		FILE30 = open( listpath30 ,"r")
		listfiles30 = FILE30.readlines()
		FILE30.close()


		# for line in filelist.txt

		for line in listfiles30:
		# make values
		      fid2, fsize2, fmd5, fdate_obs, fzip = line.split(' ')
		      fid=fid2.strip()
		      fsize=int( fsize2.strip() )
		      fmd5 = fmd5.strip().upper()

		      if fzip == 'zipfz' :
		      	zipfzCount += 1
		      if fzip == 'fz' :
		      	fzCount += 1
		      if fzip == 'fits' :
		      	fitsCount += 1
			
		      	
		      filepath2 = cwd + '/' + fid + '.fits'
			

		#								print 'chkfits filepath2' + filepath2 									
		      mGet = False
		# check Not os.path.exists
		      if not os.path.exists( filepath2 ):

		#			      mGet = True
			      missCount = missCount + 1
			      print( "Missing: "+filepath2 + " failCount: " + str( failCount ) )
		#
		# else check filebytes
		      else:
			cmd = 'md5sum '+filepath2
			fv = os.popen(cmd).read().strip()
			realMD5 = fv.split(' ')
			actualMD5 = realMD5[0]
			actualMD5=actualMD5.strip().upper()
			if not fmd5 == actualMD5:
			      failCount = failCount + 1
			#				      mGet = True

			      print( "* MD5s NoMatch *: " + filepath2  + " failCount: " + str( failCount ) )
			      print( fmd5 + " != " + actualMD5 )
			else:
			      print( "MD5s Match: "+ fid + " == " + actualMD5 )
			      
			goodCount=goodCount + 1

	print "\n-- MD5 Check Summary -- " 
	print( str( missCount ) + " = Missing Files" )
	print( str( failCount ) + " = MD5-Check Fails" )
	print( str( goodCount ) + " = MD5-Check Confirmed" ) 

	return ( missCount, failCount, goodCount )

def main():

	version='2017-07-26 unpack.py'

# os information

	mfail = 'F'
	
	print( "*" )
	print( "STARS 2 Unpack ("+ version + ")" )
	print( "- Environment -" )
#	print( "*" )

	uname2=os.uname()
	uname=uname2[0]
	print( " OS: "+uname )
	
	cwd=os.getcwd()
	print( "cwd: "+cwd )

# MacOS has no md5sum or md5sum -c or wget - so Fail

	if uname == 'Darwin':

# took md5sum off for Darwin with wget 2016-01-09 - currie

#		cmd = "which md5"

#		txt1 = os.popen( cmd ).read().strip()
#		returntxt = txt1.split()

#		if len( returntxt ) == 0 :
#			mfail = 'T'
#		else:
#			print("Darwin found md5 @ %s - OK" % ( returntxt ) )

		cmd = "which wget"

		txt1 = os.popen( cmd ).read().strip()
		returntxt = txt1.split()

		if len( returntxt ) == 0 :
		
			mfail = 'T'
			
		else:
			print("Darwin found wget @ %s - OK" % ( returntxt ) )

# DF local drive - print disk capacity

	f=os.statvfs(cwd)
	blocksize=f[statvfs.F_FRSIZE]
	freeblocks=f[statvfs.F_BAVAIL]
	diskfree=blocksize*freeblocks
	df=BytestoText(diskfree)
	print( "df: "+df+"\n" )

	if mfail == 'T' :
	
		print("MacOS-Darwin has no wget installed. Please use UNIX/Linux or install wget!")
		
# 1st Pass - query.dat information

	qanon='anon-OK'
	
	datafile='/zadmin/query.dat'
	fullpath=cwd+datafile
	if os.path.exists(fullpath):
		print( "\n" )	
		print( "1st-Pass Query Data File exists at: "+fullpath+". reading ..." )
		g=open(fullpath,'r')

		for line in g:
			querydata=line.split()
			qidno=querydata[0]
			qfiles=querydata[1]
			qsize=querydata[2]
			qcompsize=querydata[3]
			quser=querydata[4]
			qpwd=querydata[5]
			qdir=querydata[6]
			qanon=querydata[7]
			qinstr=querydata[8]
			qdate=querydata[9]

			print( "First Pass query.dat" )				
			print( "Query IDNo: "+str(qidno) )			
			print( "Query Files: "+str(qfiles) )			
			print( "Query Size: "+BytestoText(qsize) )					
			print( "Compressed Size: "+BytestoText(qcompsize) )					
			print( "Download Anonymous: "+qanon )					
			print( "Instrument: "+qinstr )					
			print( "Date: "+qdate )	
		
		g.close()			
						
	else:
		print( "1st-Pass Query Data File NOT EXIST: "+fullpath+". change directory?" )
		qidno=0
		qfiles=0
		qsize=0
		qcompsize=0
		quser=""
		qpwd=""
		qdir=""
		qanon='anon-NG'
		qinstr=''
		qdate='0000-00-00'
		
		mfail='T'
						
# move S2query.tar to ./zadmin

	datafile='/S2Query.tar'
	fullpath=cwd+datafile
	newpath=cwd+'/zadmin/S2Query.tar'
	if os.path.exists(fullpath):
		print( "Moving ./S2Query.tar to ./zadmin/S2Query.tar" )
		shutil.move(fullpath,newpath)



# Choose Network Route and protocol

	print( "-- Choose Network and Archive --" )
#			print 'Network Routing Choices\n'
	print( "1. Hawaii Americas EU - UH network from Hawaii Archive - http" )
	print( "2. Hawaii Americas EU - UH network from Hawaii Archive - https" )
	print( "3. Japan Asia AU - University network from Mitaka Archive - http" )
	print( "4. Japan Asia AU - University network from Mitaka Archive - https" )
	print( "5. Mitaka NAOJ Campus - ADC network from Mitaka Archive - http" )
	print( "6. Subaru Internal - STN network from Hawaii Archive - http" )
	print( "7. Subaru Internal - STN network from Hawaii Archive - https" )
#	print( "8. FITS Corrections - ./zcorrex/*.crx" )

#	checkpath=cwd+'/zadmin/zip-uhn.txt'
#	if os.path.exists( checkpath ):
#		print( "8. TEST-ExpZip - Hawaii Americas EU - UH - Hawaii Archive - http (.tar.bz2)" )

#	checkpath=cwd+'/zadmin/zip-uhns.txt'
#	if os.path.exists( checkpath ):
#		print( "9. TEST-ExpZip - Hawaii Americas EU - UH - Hawaii Archive - https (.tar.bz2)" )

#	checkpath=cwd+'/zadmin/zip-oc3n.txt'
#	if os.path.exists( checkpath ):
#		print( "10. TEST-ExpZip - Subaru Internal - STN - Hawaii Archive - http (.tar.bz2)" )
		
#	checkpath=cwd+'/zadmin/zip-oc3ns.txt'	
#	if os.path.exists( checkpath ):
#		print( "11. TEST-ExpZip - Subaru Internal - STN - Hawaii Archive - https (.tar.bz2)" )
		
		
	response2=raw_input('Select Network/Archive? [1-7]: ')
	
#	netFiles={ '1':'uhn.txt', '2':'uhns.txt', '3':'mtkn.txt', '4':'mtkns.txt', '5':'mtkin.txt', '6':'oc3n.txt', '7':'oc3ns.txt', '8':'zip-uhn.txt', '9':'zip-uhns.txt', '10':'zip-oc3n.txt', '11':'zip-ocs3ns.txt' } 
	netFiles={ '1':'uhn.txt', '2':'uhns.txt', '3':'mtkn.txt', '4':'mtkns.txt', '5':'mtkin.txt', '6':'oc3n.txt', '7':'oc3ns.txt', '8':'fitscorrex.txt' } 

	netfile=netFiles[ response2 ]
	
	if len( netfile ) == 0:
		netfile='uhn.txt'

	print( "netfile: "+netfile )	
	
#	if response2=='1':
#		netfile='uhn.txt'
#	if response2=='2':
#		netfile='uhns.txt'
#	if response2=='3':
#		netfile='mtkn.txt'
#	if response2=='4':
#		netfile='mtkns.txt'
#	if response2=='5':
#		netfile='mtkin.txt'
#	if response2=='6':
#		netfile='oc3n.txt'
#	if response2=='7':
#		netfile='oc3ns.txt'

#	if response2=='8':
#		netfile='zip-uhn.txt'
#	if response2=='9':
#		netfile='zip-uhns.txt'
#	if response2=='10':
#		netfile='zip-oc3n.txt'
#	if response2=='11':
#		netfile='zip-oc3ns.txt'
		
#			response3=raw_input('Check SSL certificate? [N/Y]')
	mnocheck=' --no-check-certificate'

# download updated wget input file


# default to Not Updateing unpack,py

	response20 = 'N'
	

# dont update unpack.py

#	testing = 'twin'

	testing = 'No'
	
	if testing == 'twin' :
		
		response10=raw_input('Confirm unpack.py MD5?  Usually, select N (y/n): ')
	
	
		if response10 == 'y' or response10 == 'Y' :


			md5fail = 'F' 
			md5txt = ''

			if uname == 'Darwin':

		# took md5sum off for Darwin with wget 2016-01-09 - currie

				cmd = "which md5"

				txt1 = os.popen( cmd ).read().strip()
				returntxt = txt1.split()

				if len( returntxt ) == 0 :

					md5fail = 'T'

				else:
					print("Darwin found md5 @ %s - OK" % ( returntxt[0] ) )

					md5txt = returntxt[0] + ' -r'


			else:

				cmd = "which md5sum"

				txt1 = os.popen( cmd ).read().strip()
				returntxt = txt1.split()

				if len( returntxt ) == 0 :

					md5fail = 'T'

				else:

					print("*NIX found md5sum @ %s - OK" % ( returntxt ) )
					md5txt = returntxt[0] + ' -b'



			if md5fail == 'F' and len( md5txt ) > 0 :

				newdir=cwd + '/zadmin/'

				os.chdir( newdir )

				domain = getDomain ( netfile )

				response20 = 'N'

				if not domain == 'none' :

					md5filename = newdir + 'unpack.md5'

					if os.path.exists( md5filename ):

						os.remove( md5filename )

					netfileInput = domain + qdir + "/zadmin/zadmin/unpack.md5"											

					cmd='wget --user='+quser+' --password='+qpwd+mnocheck+ " " +netfileInput+" -a ./transfer.log"

					os.system(cmd)

					if os.path.exists( md5filename ) :

						gmd5=open( md5filename,'r' )

						for line in gmd5 :

							md5data = line.split()
							md5value = md5data[0].strip()

						gmd5.close()

						cmd = md5txt + ' ./unpack.py'

						txt1 = os.popen( cmd ).read().strip()

						returntxt = txt1.split()

						if len( returntxt ) > 0 :

							md5real = returntxt[0].strip()

							if md5real.upper() == md5value.upper() :

								print( "unpack.py == unpack.md5, current unpack.py script may continue.\n" )


							else:

								response20 = 'Y'		

					else :
					
						print( "unpack.py MD5 from STARS is not available. current unpack.py script may continue.\n" )


# dont uupdate unpack.py

#	response20 = 'N' | this is above!
	
	if response20 == 'Y':
				
		response21=raw_input('Your unpack.py requires updating. Update Now? (y/n): ')
		
		if response21 == 'Y' or response21 == 'y': 

	# change dir to /zadmin for the netfile download


			mfail = 'T' 
								
			newdir=cwd + '/zadmin/'

			os.chdir( newdir )

			domain = getDomain ( netfile )

			if not domain == 'none' :

				unpackfilename = newdir + 'unpack.py'
			
				if os.path.exists( unpackfilename ):

					os.remove( unpackfilename )
					
#			cmd='wget -N --user='+quser+' --password='+qpwd+mnocheck+ " " +netfileInput+" -a ./transfer.log"

				print("Please restart unpack.py after this download-update!\n************************")

				netfileInput = domain + qdir + "/zadmin/zadmin/unpack.txt"
															
				cmd='wget --user='+quser+' --password='+qpwd+mnocheck+ " " +netfileInput+" -a ./transfer.log"
				os.system(cmd)

				os.rename('./unpack.txt', './unpack.py')

				cmd='chmod 755 ./unpack.py'

				os.system( cmd )				
				
				os.chdir(cwd)
				
				os.execv(__file__,sys.argv)
				

# change dir back											
	os.chdir(cwd)


# if we did not update unpack.py

	if mfail == 'F':


# 2nd pass - query information after update

		qanon='anon-OK'

		datafile='/zadmin/query.dat'
		fullpath=cwd+datafile
		if os.path.exists(fullpath):
			print( "\n" )
			print( "Query Data File exists at: "+fullpath+". reading ..." )
			g=open(fullpath,'r')
			for line in g:
				querydata=line.split()
				qidno=querydata[0]
				qfiles=querydata[1]
				qsize=querydata[2]
				qcompsize=querydata[3]
				quser=querydata[4]
				qpwd=querydata[5]
				qdir=querydata[6]
				qanon=querydata[7]
				qinstr=querydata[8]
				qdate=querydata[9]

				print '2nd Pass - updated query.dat'		

				print( "Query IDNo: "+str(qidno) )			
				print( "Query Files: "+str(qfiles) )			
				print( "Query Size: "+BytestoText(qsize) )					
				print( "Compressed Size: "+BytestoText(qcompsize) )					
				print( "Download Anonymous: "+qanon )					
				print( "Instrument: "+qinstr )					
				print( "Date: "+qdate )					


		else:
			print( "Query Data File NOT EXIST: "+fullpath+". change directory?" )
			qidno=0
			qfiles=0
			qsize=0
			qcompsize=0
			quser=""
			qpwd=""
			qdir=""
			qanon='anon-NG'

			qinstr=''
			qdate=''
			mfail='T'


		print( "\n- Count Current Files in Download Directory -\n" )

	# check files downloaded
		if qinstr == 'HSCSSP':
			fullpath = cwd + '/' + qdate + '/'
		else:
			fullpath = cwd + '/'


		if os.path.exists( fullpath ):

			os.chdir ( fullpath )

			cmd = 'ls -al *.fits | wc -l'								
			fv = os.popen(cmd).read().strip()
			fv = fv.strip()
		else:
			fv = '0'

		os.chdir ( cwd )

	#	if int( fv ) > 0 :

	# calculate remaining files from ( query.dat - filecount )

		remainFiles = int( qfiles ) - int( fv )

		print( str(fv) + " FITS Files exist in " + cwd )
		print( str(qfiles) + " Total FITS Files are listed for this Query." )
		print( str(remainFiles) + " FITS Files Missing to be downloaded.\n" )

	# is anonymous download allowed? anon-NG - enter personal login/pw

		if qanon == 'anon-NG' :

			print( "No Anonymous Download allowed: " + qanon )	
			response50=raw_input('Enter Personal STARS Login: ')
			quser=response50.strip()
			response51=raw_input('Enter Personal STARS Password: ')
			qpwd=response51.strip()

	# if existing files > 0, then confirm MD5s?
		

#		if int( fv ) > 0 :

		noMissingMD5 = 'T'
		
		if noMissingMD5 == 'F' :
		
			existingFiles=str( fv )	
			confirmText= "Confirm MD5s on " + existingFiles + " Existing Files?"
			response40=raw_input('Confirm MD5s on existing Files? (y/n)')
			if response40 =='Y' or response40 == 'y' :
				missCount, failCount, goodCount = checkMD5s( qdir, netfile, quser, qpwd, mnocheck )

	# if remaining files in download directory > 0

			if remainFiles > 0 :

				response20=raw_input('List Missing files? (y/n)')

				if response20=='y' or response20=='Y':

					listpath30 = cwd+'/zadmin/filelist.txt'
					if os.path.exists( listpath30 ):

			# open filelist.txt
						FILE30 = open( listpath30 ,"r")
						listfiles30 = FILE30.readlines()
						FILE30.close()

						missingSeq = 0			

						for line in listfiles30:
		# make values
							fid2, fsize2, fmd5, fdate_obs =line.split(' ')
							fid=fid2.strip()
							fsize=int( fsize2.strip() )
							fmd5 = fmd5.strip()
							fmd5 = fmd5.upper()

							if qinstr == 'HSCSSP':
								filepath2 = cwd + '/' + qdate + '/' + fid + '.fits'
							else:
								filepath2 = cwd + '/' + fid + '.fits'

	#						filepath2 = cwd + '/' + fid + '.fits'
		#
		#					print 'chkfits filepath2' + filepath2 									
		#					mGet = False
		# check Not os.path.exists
							if not os.path.exists( filepath2 ):
								missingSeq = missingSeq + 1							
								print( str(missingSeq) +" - Missing: " + fid + ".fits" )

					if missingSeq > 0:

						print( "\n* To Download the Above-Listed Missing Files Only:" )
						print( "* choose 'Continue Wget Preparation?' = Y" )
		#				print "* choose 'Network Route' number"
						print( "* choose 'Select FrameID Range to Download?' = N" )
						print( "* choose 'Download Missing/Error FITS Files?' = Y" )
						print( "* choose 'Confirm MD5s values for Existing Files?' = N" )
						print( "* choose 'Fix MD5 Errors - Delete Errors & re-Download?' = N\n" )
		#		else:
		#			mfail = 'T'


	# confirm disk space wget
	#	print '\n'
	#	print '-- Transfer DiskSpace Check --'
	##	print '\n'

	#	qs=BytestoText(qcompsize)
	#	print 'diskfree: ' + str( diskfree )
	#	print 'qcompsize: ' + str( qcompsize )
	#	if int( diskfree ) > int( qcompsize ):
	#		print 'You have free space to transfer: '+qs+' / '+df
	#	else:
	#		print 'You have NO free space to transfer: '+qs+' / '+df
	#		print 'You have NO free space to transfer: '+qs+' / '+df
	#		mfail='T'


	#	print( "above mfail: "+mfail )	
	# wget



	wgetFound, wgetFoundText = wGetRunning()
	
	if wgetFound == True:
	
		mfail = 'T'

		print("\n************************")
		print("Critical Error! You have a wget process currently-running!")
		print("This is a common problem. 'ps -ef | grep wget' reports:")
		print( wgetFoundText )
		print("The unpack.py script runs without a progress bar and downloads with no notice.")
		print("Look for files in the download directory, and if new files are not appearing\nregularly, then kill the running wget process and restart unpack.py.")
		print("Otherwise - if files are appearing in the download dir regularly, then let the currently-running wget finish.")
		print("************************")

	if mfail == 'F' and not netfile == 'fitscorrex.txt' :
	
		start_time = time.time()
#		print( "past mFail = T" )
#	

		response1=raw_input('Prepare to Download? (y/n)')


		zipfzCount = 0
		fzCount = 0
		fitsCount = 0
		
		listpath30 = cwd+'/zadmin/filelist.txt'

		if os.path.exists( listpath30 ):

			# open filelist.txt
			FILE30 = open( listpath30 ,"r")
			listfiles30 = FILE30.readlines()
			FILE30.close()


			# for line in filelist.txt

			for line in listfiles30:
			# make values
			      fid2, fsize2, fmd5, fdate_obs, fzip = line.split(' ')
			      fid=fid2.strip()
			      fsize=int( fsize2.strip() )
			      fmd5 = fmd5.strip().upper()
			      fzip = fzip.strip()

			      if fzip == 'zipfz' : 
		      		zipfzCount += 1
				
			      if fzip == 'fz' :
		      		fzCount += 1
				
			      if fzip == 'fits' :
		      		fitsCount += 1
		

		if qinstr == 'HSC' and ( zipfzCount > 0 ):
		
			responseZip=raw_input('HSC Compressed (FPACK) found - Download Compressed? (y/n)')
		
		else: 
			responseZip = 'N'

#		responseZip = 'N'
		
			
		
		if response1=='y' or response1=='Y':



			start_time = time.time()
			
# write inbound.txt with domain + qdir + filelist files +  .fit
		
			domain = getDomain ( netfile )

			filelist = '/zadmin/filelist.txt'
			fullpath = cwd + filelist
			outfilelist = '/zadmin/inbound.txt'
			fullpathout = cwd + outfilelist

			f = open( fullpathout,'w')

#			print("fullpath: "+fullpath)
#			print("fullpathout: "+fullpathout)
		
			loglist=[ 'TSC', 'END', 'TAS', 'TAD', 'STA', 'ENV', 'QRE', 'QCA' ]			
			
			if os.path.exists( fullpath ) :

#				print("fullpath Exists: "+fullpath)
			
				g=open( fullpath, 'r')				
				listfiles=g.readlines()
				g.close()
#				print("listfiles: "+str(listfiles))				
				for line in listfiles:

					filedata = line.split()
					qframeid = filedata[0]
					qfprefix = qframeid[0:3]
					if len ( filedata ) > 4:
					
						zipStatus = filedata[4]
					else:
						zipStatus = 'fits'
					
#					print('writing: domain-' + domain + ' qdir-'+ qdir + ' qframeid-'+ qframeid +	".fits")

					if qfprefix in loglist or zipStatus == 'log' :	
										
						f.write( domain + qdir + "/" + qframeid + "\n" )
						
					else:
					
#						if responseZip == 'Y' or responseZip == 'y' :						
#							f.write( domain + qdir + "/zip/" + qframeid + ".fits.fz\n" )
						#						else: 					
#							f.write( domain + qdir + "/" + qframeid + ".fits\n" )

						if zipStatus == 'fits' :
												
							f.write( domain + qdir + "/" + qframeid + ".fits\n" )
							
						if zipStatus == 'zipfz' :
						
							if responseZip == 'Y' or responseZip == 'y' :
						
								f.write( domain + qdir + "/zip/" + qframeid + ".fits.fz\n" )

							else :
								
								f.write( domain + qdir + "/" + qframeid + ".fits\n" )
							
						if zipStatus == 'fz' :
						
							f.write( domain + qdir + "/" + qframeid + ".fits.fz\n" )
						
			else:
			
				print("fullpath Not Exists: "+fullpath)

			f.close()
			
			netfile = 'inbound.txt'
		
#			print '\n'				
			

# Start / End Frame_ID

			response6=raw_input('Start Download from one FrameID? (n = ALL FrameIDs) (y/n): ')

			if response6 == 'y' or response6 == 'Y' :

				response7=raw_input('Enter Start FrameID: ')

				if len( response7 ) == 12 :

					response9=raw_input('Enter End FrameID (blank for All): ')

					endFrame = ''

					if len( response9 ) == 12 :
						endFrame = response9

					restartFrameID = response7
					inputpath2=cwd+'/zadmin/' + netfile	
					today = datetime.datetime.today()
					todayString=today.strftime('%y%m%d')

					netfile2= netfile + "." + todayString	

					inputpath3=cwd+'/zadmin/' + netfile2
					print( "inputpath2: " + inputpath2 + " inputpath3: " + inputpath3 )									
					if os.path.exists( inputpath2 ):

						FILE1 = open( inputpath2 ,"r")
						listfiles2 = FILE1.readlines()
						FILE1.close()

						FILE2 = open( inputpath3 ,"w")

						seq = 0
						startHere = 'No'
						endHere = 'No'

						for line in listfiles2:

							seq += 1

#							print str(seq) + ' '+ line

							if response7 in line:

								print("* Start FrameID Found: "+response7 + " startHere=Yes" )

								startHere = 'Yes'

							if startHere == 'Yes' and endHere == 'No':

								FILE2.write( line )

							if endFrame != '' and endFrame in line:

								print("* End FrameID Found: "+ endFrame + " endHere=Yes" )

								endHere = 'Yes'




						FILE2.close()

						if startHere == 'Yes':

							print("Selected Network Routing is: "+response2+": "+netfile2+" "+mnocheck+"\n")
							print("Transfer log is appended to ./zadmin/transfer.log\n\n")
							print("FITS transferring now, please wait ...\n")

							if qinstr == 'HSCSSP':
								cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ./zadmin/'+netfile2+' -a ./zadmin/transfer.log' + ' -P '+qdate
							else: 
								cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ./zadmin/'+netfile2+' -a ./zadmin/transfer.log'
								
							os.system(cmd)
						else:

							print("Start FrameID NotFound: " + response7 + "\n" )



				else:
					resp7len = len( response7 )
					print("Start FrameID is not 12 characters: (" + str( resp7len ) + ") - " + response7 )
					
						
# Not Start/End, so eitiher a) Missing/MD5 Fails or b) HSC-SSP Provide Date or c) Download All

			else:


# a) Missing/MD5 Fails		
				response10=raw_input('Download Missing/MD5Fails files ONLY? (n = Download ALL) (y/n): ')
			
				if response10 == 'y' or response10 == 'Y' :

					response21=raw_input('Confirm MD5s of Existing Files? (n = only check Missing files)(y/n): ')
					
					response11=raw_input('Fix MD5 Errors - Delete Errors & re-Download? (n = NoDelete/NoDownload) (y/n): ')
					
						
					listpath30 = cwd+'/zadmin/filelist.txt'

					if os.path.exists( listpath30 ):

# open filelist.txt
						FILE30 = open( listpath30 ,"r")
						listfiles30 = FILE30.readlines()
						FILE30.close()

# open existing netfile

						inputpath2=cwd+'/zadmin/' + netfile	
						today = datetime.datetime.today()
						todayString=today.strftime('%y%m%d')
# open new netfile_YYMMDDD
						netfile2= netfile + "." + todayString	

						inputpath3=cwd+'/zadmin/' + netfile2

#						print 'chkfits inputpath2: ' + inputpath2 + ' inputpath3: ' + inputpath3									
						FILE3 = open( inputpath3 ,"w")


						if os.path.exists( inputpath2 ):

# read netfile into array
							FILE1 = open( inputpath2 ,"r")
							listfiles2 = FILE1.readlines()
							FILE1.close()
							
							failCount = 0
# for line in filelist.txt
							for line in listfiles30:
# make values
#								fid2, fsize2, fmd5, fdate_obs =line.split(' ')
								filedata=line.split(' ')
								
								fid2 = filedata[0]
								fsize2 = filedata[1]
								fmd5 = filedata[2]
								fdate_obs = filedata[3]
								
								if len ( filedata ) > 4 :
								
									fzipStatus = filedata[4]
									
								else:
								
									fzipStatus = 'fits'
									
								
								fid=fid2.strip()
								
								fsize=int( fsize2.strip() )
								fmd5 = fmd5.strip().upper()
								

								if qinstr == 'HSCSSP':
								
									filepath2 = cwd + '/' + qdate + '/' + fid + '.fits'
								else:
									filepath2 = cwd + '/' + fid + '.fits'
								
#								print 'chkfits filepath2' + filepath2 									
								mGet = False
# check Not os.path.exists
								if not os.path.exists( filepath2 ):

									mGet = True
									failCount = failCount + 1
									print("Missing: "+filepath2 + " failCount: " + str( failCount ) )
#
# else check filebytes
								else:
									if response21 == 'Y' or response21 == 'y' :
	# filebytes
	#									cmd = 'cksum '+filepath2
	#									fv = os.popen(cmd).read().strip()
	#									realCksum = fv.split(' ')
	#									print 'realCksum: ' + realCksum[0] + ' realCBytes: '+realCksum[1] + ' realFile: '+realCksum[2]
	#									realFSize = int( realCksum[1] )
	#									if not fsize == realFSize:

	# md5 checking									
										cmd = 'md5sum '+filepath2
										fv = os.popen(cmd).read().strip()
										realMD5 = fv.split(' ')
	#									print 'realMD5: ' + realMD5[0] + ' realMD5File: '+realMD5[1]
										actualMD5 = realMD5[0]
										actualMD5=actualMD5.strip().upper()

										if not fmd5 == actualMD5:


	# if file exists but wrong size - unlink and wget again				
											if response11 == 'Y' or response11 == 'y' :				
												cmd = 'unlink '+ filepath2
												os.system(cmd)

											failCount = failCount + 1
											mGet = True

											print( "* MD5s NoMatch *: " + filepath2  + " failCount: " + str( failCount ) )
											print( fmd5 + " != " + actualMD5 )

										else:
											print("MD5s Match: "+ fid + " == " + actualMD5 )
										

								if mGet == True :
									for line2 in listfiles2 :
										if fid in line2 :
											FILE3.write( line2 )
#											print 'writing line2: '+line2
											
																	
																 
						FILE1.close()
						FILE3.close()
						FILE30.close()
						
						print("Total Missing/MD5Fails Count: " + str( failCount ) )
						
						if ( response11 == 'Y' or response11 == 'y' ) and failCount > 0 :
																								
							print( "Selected Network Routing is: "+response2+": "+netfile2+" "+mnocheck+"\n" )
							print( "Transfer log is stored as ./zadmin/transfer.log\n\n" )
							print( "transferring now, please wait ...\n" )

							if qinstr == 'HSCSSP':
								cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ./zadmin/'+netfile2+' -a ./zadmin/transfer.log' + ' -P '+qdate
							else:
								cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ./zadmin/'+netfile2+' -a ./zadmin/transfer.log'
							
							os.system(cmd)
								

				else:
				
 # b) HSC-SSP Provide Date
					if qinstr == 'HSCSSP':

						msg = "HSC-SSP: Enter Date to DL {lastDate: " + qdate + "}:"
											
						response22=raw_input( msg )

						if len( response22 ) == 10  and response22[4:5] == '-' and response22[7:8] == '-':
						
							entryDate = response22

# get netfile name 
#							inputpath2=cwd+'/zadmin/' + netfile	
#							today = datetime.datetime.today()
#							todayString=today.strftime('%y%m%d')
#
#							netfile2 = netfile + "." + todayString	
# add todayString %Y-%m-%d to netfile name


							dlFileList = entryDate + '.txt'
							dlFileList2 = entryDate + '.inp'

							dlFileListPath = cwd + './zadmin/' + dlFileList
							dlFileListPath2 = cwd + './zadmin/' + dlFileList2
													
							if os.path.exists( dlFileListPath ):

								cmd = 'rm -f '+dlFileListPath
								os.system( cmd )

							if os.path.exists( dlFileListPath2 ):

								cmd = 'rm -f '+dlFileListPath2
								os.system(cmd)
								
							domain=getDomain ( netfile )
							
							dateFileList= domain + qdir + '/' + entryDate + '/' + dlFileList

							dlfilepath2 = cwd + '/zadmin/'
							
							os.chdir( dlfilepath2 )

							cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' ' + dateFileList + ' -a ./zadmin/transfer.log' + ' -P '+qdate

							os.chdir( cwd )
							
							
							if os.path.exists( dlfilepath2 + dateFileList ) :
							
								FILE1 = open( dlfilepath2 + dateFileList ,"r")
								listfiles3 = FILE1.readlines()
								FILE1.close()								

								FILE2 = open( dlfilepath2 + dateFileList2 ,"w")
								
								for line in listfiles3:
									
									linearray = line.split(' ')
									dlframe = linearray[0]
									writeline = domain + qdir + '/' + entryDate + '/' + dlframe
									FILE2.write(writeline)
								
								FILE2.close()

								print( "Selected Network Routing is: "+response2+": "+netfile2+" "+mnocheck+"\n" )
								print( "Data is downloaded to: ./" + entryDate + "\n" )
								print( "Transfer log is appended to ./zadmin/transfer.log\n\n" )
								print( "transferring now, please wait ...\n" )
																	
								cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ./zadmin/'+dateFileList2+' -a ./zadmin/transfer.log' + ' -P '+entryDate

								os.system(cmd)
									
							


# if HSCSSP but One Date and One Date Found in netfile, download to ./{One Date}, like ./2015-05-13

							if startHere == 'Yes':

								print( "Selected Network Routing is: "+response2+": "+netfile2+" "+mnocheck+"\n" )
								print( "Data is downloaded to: ./" + entryDate + "\n" )
								print( "Transfer log is appended to ./zadmin/transfer.log\n\n" )
								print( "transferring now, please wait ...\n" )

								cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ./zadmin/'+dateFileList+' -a ./zadmin/transfer.log' + ' -P '+entryDate

								os.system(cmd)
# if HSCSSP but all dates
						else:
							print 'HSC-SSP Downloads only allow download of One Date in format YYYY-MM-DD'						
# download all, Not HSCSSP
					
					else:  
					
					
						print( "Selected Network Routing is: "+response2+": "+netfile+" "+mnocheck+"\n" )
						print( "Transfer log is stored as ./zadmin/transfer.log\n\n" )
						print( "transferring now, please wait ...\n" )					
						cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ./zadmin/'+netfile+' -a ./zadmin/transfer.log'
						os.system(cmd)

		else:

			print( "You chose to NOT continue Wget Preparation\n" )



	elapsed_time = int( time.time() - start_time )
	
	print ("Elapsed download in seconds: " + str( elapsed_time ) )


	if qinstr == 'HSCSSP':
		filepath2 = cwd + '/' + qdate + '/'
	else:
		filepath2 = cwd + '/'


#	if os.path.exists( filepath2 ) :
#		os.chdir ( filepath2 )
#	else:
#		os.chdir ( cwd )







	correxfile = 'fitscorrex.txt'
	
	inputcorrex  = cwd + '/zadmin/' + correxfile
				
	if os.path.exists ( inputcorrex ) :

# open correxlist.txt
		FILE30 = open( inputcorrex ,"r")
		listfiles30 = FILE30.readlines()
		FILE30.close()
		
		correxseq = 0
		
		for line in listfiles30:
		
			correxseq += 1
			
		if correxseq > 0 :
		
			print ("Corrections Exists: " + str ( correxseq ) ) 
			
			responseCorrex = raw_input( "Download Corrections? y/n " )
			
			if responseCorrex == 'Y' or responseCorrex == "y" :
			

				inputpath2=cwd+'/zadmin/' + netfile
				today = datetime.datetime.today()
				todayString=today.strftime('%y%m%d')
		# open new netfile_YYMMDDD
				netfile2= netfile + ".Crx." + todayString

				inputpath3= cwd + '/zadmin/' + netfile2 

#						print 'chkfits inputpath2: ' + inputpath2 + ' inputpath3: ' + inputpath3									
				FILE3 = open( inputpath3 ,"w")

				for line in listfiles30:
		# make values
		#								fid2, fsize2, fmd5, fdate_obs =line.split(' ')
					filedata=line.split(' ')

					fid2 = filedata[0]
					fsize2 = filedata[1]
					fmd5 = filedata[2]
					fdate_obs = filedata[3]
					
					
					FILE3.write( domain + qdir + "/zcorrex/" + fid2 + ".crx\n" )
					

				FILE3.close()
				
				filepath2 = cwd + '/zcorrex/'
				
				print( 'filepath2: ' + filepath2 )
				
				os.chdir ( filepath2 )
				
				cmd='wget --user='+quser+' --password='+qpwd+mnocheck+' -i ../zadmin/'+netfile2+' -a ../zadmin/transfer.log' 
				
				print("wgetCmd: "+cmd )
					
				os.system( cmd )
				
				print( "Corrections Download Completed" )	
				
				filelist=os.listdir( filepath2 )
				
				docorrex='F'
				
				seq=0
				
				correxfiles = len( filelist )
				
				print("Corrections Downloaded to " + filepath2 + "{FrameID}.crx : " + str( correxfiles ) )  
				
				correxseq = 0
				
				for item in filelist:
				
					correxseq += 1
				
					print( str(correxseq) + " " + item )
										
								

			else:
			
				print( "Dont Download Corrections!" )									


		else:

			print ("No Corrections in File. Corrections = " + str( correxseq ) )

	else:

		print ("Corrections File Not Exists: " + inputcorrex )


	
#	os.chdir( cwd )
#	filelist=os.listdir( cwd )
#	unzip='F'
#	seq=0
#	for item in filelist:
#		if item[-4:] == '.bz2' :
#			seq=seq+1
#			unzip='T'
#	if unzip == 'T':
#		questionString=str(seq)+" Bzip2 files found. Bunzip2? (y/n): " 
#		response51=raw_input( questionString )
#		if response51 == 'Y' or response51 == 'y' :
#			cmd = 'bunzip2 *.bz2'
#			os.system( cmd )

#	untar='F'
#	seq=0
#	for item in filelist:
#		if item[-4:] == '.tar' :
#			seq=seq+1
#			untar='T'
#	if untar == 'T':
#		questionString=str(seq)+" Tar files found. Extract Tars? (y/n): " 
#		response52=raw_input( questionString )
#		if response52 == 'Y' or response52 == 'y' :
#			cmd = 'tar -xvf *.tar'
#			os.system( cmd )



		
	print("\n")
	print("- STARS Unpack COMPLETED -")
	print("\n")

	os.chdir ( cwd )
	

if __name__ == '__main__':

	main()
